#![deny(rust_2018_idioms)]
#![deny(unused_must_use)]

/// This crate only works if either headless or graphical is enabled
mod assets;

pub fn parse_vector<T: std::str::FromStr>(s: &str) -> anyhow::Result<Vec<T>>
where
    <T as std::str::FromStr>::Err: std::fmt::Display,
{
    let mut v = Vec::new();

    for split in s.trim_start_matches('[').trim_end_matches(']').split(',') {
        let trimmed = split.trim();

        if !trimmed.is_empty() {
            v.push(trimmed.parse().map_err(|err| {
                anyhow::Error::msg(format!(
                    "Failed parsing {} for parse_vector from: {}",
                    err, s
                ))
            })?);
        }
    }

    Ok(v)
}

pub mod scene;

#[macro_use]
pub mod engine;

pub mod prelude;
